from threading import Timer
from requests.exceptions import Timeout
import sys
import datetime
import time

try:
    import thread
except ImportError:
    # for py3
    import _thread as thread


class OUT_TIME(object):
    req_timeout = 30

    @property
    def out_time(self):
        return self.req_timeout

    @out_time.setter
    def out_time(self, timeout):
        self.req_timeout = timeout


def accident_protected(PType):
    def warps(func):
        def deco(*args, **kwargs):
            result = None
            try:
                result = func(*args, **kwargs)
            except PType:
                import traceback
                print("Error in protected")
                print(traceback.format_exc())
            finally:
                return result

        return deco

    return warps


def time_out(interval):
    def wraps(func):
        def quite_function(fn_name):
            # print('{0} took too long'.format(fn_name), sys.stderr)
            thread.interrupt_main()  # raises KeyboardInterrupt

        def deco(*args, **kwargs):
            outer = interval.out_time
            timer = Timer(outer, quite_function, args=(func.__name__,))
            timer.start()
            try:
                resp = func(*args, **kwargs)
            except KeyboardInterrupt:
                raise Timeout("Request time out, max time:[%ss]" % outer)
            finally:
                timer.cancel()
            return resp

        return deco

    return wraps


standard_format_str = "%Y-%m-%d %H:%M:%S"


def add_days(run_date, days):
    new_date = run_date + datetime.timedelta(days=days)
    return new_date


def timestamp_to_datetime(timestamp):
    time_local = time.localtime(timestamp)
    dt = time.strftime(standard_format_str, time_local)
    return dt


def datetime_to_timestamp(date_time):
    return time.mktime(date_time.timetuple())


def string_to_datetime(string, format_str):
    try:
        return datetime.datetime.strptime(string, format_str)
    except Exception:
        return None


def date_format(date, format_str):
    return date.strftime(format_str)


def now():
    t = datetime.datetime.now()
    st = date_format(t, standard_format_str)
    r = datetime.datetime.strptime(st, standard_format_str)

    return r


def time_see(func):
    def deco(*args, **kwargs):
        b = time.time()
        result = func(*args, **kwargs)
        a = time.time()

        utime = round(a - b, 3)
        print("----Use: %ss" % utime)
        return result

    return deco


from Core.JobBase import JobBase
import json
from Test.TestDao import TestDao


class TestAction(JobBase):
    api_key = ''

    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)

    def initialize(self):
        self.maxHourlyPageView = 1000000
        self.proxy = [self.PROXY_SQUID_US_3]
        if self.debug():
            self.proxy = [self.LOCALHOST]
        self._dao = TestDao(run_id=self.run_id,run_date=self.run_date, log=self.log)

    def on_run(self):
        try:
            self.log.info("%s has started" % self.__class__.__name__,
                          "jobID:[%s]" % self.job_id)
            self.initialize()
            self.api_key = ''
            self.test()
            self.log.info("%s has finished" % self.__class__.__name__,
                          "jobID:[%s]" % self.job_id)
        except Exception as e:
            self.log.error("Unexpected/Unhandled Error", str(e))

    # def get_popularity(self, place_id):
    #     response = populartimes.get_id(self.api_key, place_id)
    #     # self.
    #     # for each in

    def test(self):
        url = 'http://ip-api.com/json'
        page = self.download_page(url, self.init_http_manager(default_header=True), validate_str_list=[''])
        self._dao.save(json.loads(page))

    pass

if __name__ == '__main__':
    t = TestAction()
    t.on_run()


import os
import datetime
import csv
from .Log import Log


class DaoBase(object):
    mongo_client = None
    rows = []

    def __init__(self, **kwargs):
        self._run_date = kwargs.get("run_date", datetime.datetime.now())
        self.log = kwargs.get('log', Log(self.__class__.__name__))
        self._pool_name = None

    def save(self, source_block):
        pass

    def connect_to_mongo(self):
        import pymongo
        mongo_config = {
            "host": os.getenv('MONGO_HOST', 'localhost'),
            "port": os.getenv('MONGO_PORT', 27017),
            "user": os.getenv('MONGO_USER', None),
            "password": os.getenv('MONGO_PASSWORD', None)
        }

        self.mongo_client = pymongo.MongoClient("mongodb://{user}:{password}@{host}:{port}".format(**mongo_config).replace('None:None@', ''))

    def connect_to_mysql(self):
        pass

    def parse(self, source_block):
        pass

    def export_to_csv(self, file):
        fields = self.rows[0].keys()
        with open(file, 'w', encoding='utf-8') as output_file:
            dict_writer = csv.DictWriter(output_file, fields)
            dict_writer.writeheader()
            dict_writer.writerows(self.rows)





# to find dict in a dict
def find_dict(key, dictionary):
    for k, v in dictionary.items():
        if k == key:
            yield v
        elif isinstance(v, dict):
            for result in find_dict(key, v):
                yield result
        elif isinstance(v, list):
            for d in v:
                if isinstance(d, dict):
                    for result in find_dict(key, d):
                        yield result

# to find dict in a dict key = value
def find_dict2(key, val, dictionary):
    for k, v in dictionary.items():
        if k == key and v == val:
            yield dictionary
        elif isinstance(v, dict):
            for result in find_dict2(key, val, v):
                yield result
        elif isinstance(v, list):
            for d in v:
                if isinstance(d, dict):
                    for result in find_dict2(key, val, d):
                        yield result




class Proxy:
    using_proxy = None

    def __init__(self, proxy_point=''):
        self.proxy_poing = proxy_point
        if proxy_point:
            self.using_proxy = {"https": "https://" + proxy_point, 'http:': "http://" + proxy_point }











import pandas as pd
from sqlalchemy import create_engine


class MysqlHelper:
    def __init__(self, *args, **kwargs):
        self.df = pd.DataFrame()
        self.source_mysql_config = kwargs.get('source_mysql_config')
        self.target_mysql_config = kwargs.get('target_mysql_config')

        self.source_engine= create_engine("{dr}://{user}:{password}@{host}:{port}/{database}" \
                                    .format(dr='mysql', **self.source_mysql_config))

        self.target_engine= create_engine("{dr}://{user}:{password}@{host}:{port}/{database}" \
                                          .format(dr='mysql', **self.target_mysql_config))

    def csv_to_mysql(self, input_file_name):
        self.get_data_from_csv(input_file_name)
        self.export_data_to_mysql()
        pass

    def mysql_to_csv(self, sql_input, output_file_name):
        self.get_data_from_mysql(sql_input)
        self.export_data_to_csv(output_file_name)

    def mysql_to_mysql(self, input_sql):
        self.get_data_from_mysql(input_sql=input_sql)
        self.export_data_to_mysql()

    def get_data_from_csv(self, file_name):
        self.df = pd.read_csv(file_name, index_col=None)

    def get_data_from_mysql(self, input_sql):
        with self.source_engine.connect() as conn, conn.begin():
            self.df = pd.read_sql(sql=input_sql, con=conn, index_col=None)

    def export_data_to_csv(self, target_file_name):
        self.df.to_csv(target_file_name)
        pass

    def export_data_to_mysql(self):
        table_name = self.target_mysql_config['table_name']
        with self.target_engine.connect() as conn, conn.begin():
            self.df.to_sql(table_name, conn, if_exists='append', index=False)




import datetime
import os
from .Log import Log
from .HttpManager import HttpManager
from .Proxy import Proxy


class JobBase:

    def __init__(self, *args, **kwargs):
        self.run_date = datetime.datetime.now()
        self.init_folder()
        self.log = Log(self.__class__.__name__)
        self.maxHourlyPageView = 600
        self.proxy = None
        self.job_id = 1001
        self.run_id = 1001
        self._proxy_instance = Proxy()

    def init_folder(self):
        if not os.path.exists('Log/'):
            os.mkdir('Log/')

        if not os.path.exists('Data/'):
            os.mkdir('Data/')

        if not os.path.exists('Records/'):
            os.mkdir('Records/')


    def init_http_manager(self, timeout=30, default_header=False):

        manager = HttpManager(proxy=self._proxy_instance
                              , default_header=default_header
                              , log=self.log
                              , timeout=timeout
                              , max_hourly_page_view=self.maxHourlyPageView
        )

        return manager

    def download_page(self
                      , url
                      , manager
                      , max_retry=10
                      , post_data=None
                      , validate_str_list=None
                      ):

        for _ in range(max_retry):
            resp = self.get_response(url, manager
                                     , max_retry=1
                                     , post_data=post_data)

            if validate_str_list:
                for each in validate_str_list:
                    if resp.text and each in resp.text:
                        return resp.text
                    if resp.content and each in resp.content:
                        return resp.content

    def debug(self):
        return not (os.getenv('PRODUCTION') == 'TRUE')

    def get_response(self, url
                     , manager
                     , max_retry=10
                     , post_data=None
                     ):
        retry = 0
        while retry <= max_retry:
            retry += 1
            # When retry big then 1, need be write log
            if retry > 3: manager.retry_log(url)

            resp = manager.download_page(url, post_data=post_data)
            if resp:
                return resp

        self.log.error("Retried all failed", "url => %s post_data => %s" % (url, post_data))
        return resp

    @property
    def LOCALHOST(self):
        proxy_str = '127.0.0.1:8888'
        # proxy_str = '192.168.0.109:8888'
        self._proxy_instance = Proxy(proxy_str)
        return proxy_str

    @property
    def NONE_PROXY(self):
        proxy_str = ''
        self._proxy_instance = Proxy(proxy_str)
        return proxy_str

    @property
    def PROXY_SQUID_US_3(self):
        proxy_str = ''
        self._proxy_instance = Proxy(proxy_str)
        return proxy_str

    @property
    def LOCAL_PROXY_P4(self):
        proxy_str = ''
        self._proxy_instance = Proxy(proxy_str)
        return proxy_str

    @property
    def LOCAL_PROXY_P5(self):
        proxy_str = ''
        self._proxy_instance = Proxy(proxy_str)
        return proxy_str


import os
import subprocess
from sys import platform


class S3Util:

    def __init__(self, logger):
        self.log = logger

    def exec_cmd_str(self, cmd_str, retry_count=20):

        result = -1
        i = 0
        while i < retry_count and result != 0:
            i += 1
            self.log.info('try %s ' % str(i))
            try:
                result = subprocess.check_call(cmd_str, shell=True)
            except Exception as e:
                self.log.error('failed to upload list file result :%s' % str(result), str(e))

        if result != 0:
            self.log.error("retry all failed")
        else:
            self.log.info('success')

    def upload(self, file, s3_path):
        profile = s3_path.replace('s3://', '').split('/')[0]
        assert s3_path[-1] == '/', 'the target path must be a folder'


        if platform == 'win32':  # windows
            self.log.info("change to scripts directory")
            os.chdir("C:\Python27\Scripts")
            cmd_str = 'aws s3 cp {} {} --profile {}'.format(file, s3_path, profile)

        elif platform in ['linux2', 'darwin', 'linux']:  # ubuntu
            aws_path = '/usr/local/bin/'
            cmd_str = '{}aws s3 cp {} {} --profile {}'.format(aws_path, file, s3_path, profile)
        else:
            self.log.error('unknown platform', str(platform))
            return
        self.log.info('start to upload file')
        self.exec_cmd_str(cmd_str)

    def download(self, file, s3_path):
        profile = s3_path.replace('s3://', '').split('/')[0]
        if platform == 'win32':  # windows
            os.chdir("C:\Python27\Scripts")
            cmd_str = 'aws s3 cp {} {} --profile {}'.format(s3_path, file, profile)
            self.log.info("change to scripts directory")
        elif platform in ['linux2', 'darwin', 'linux']:  # ubuntu
            aws_path = '/usr/local/bin/'
            cmd_str = '{}aws s3 cp {} {} --profile {}'.format(aws_path, s3_path, file, profile)
        else:
            self.log.error('unknown platform', str(platform))
            return

        self.log.info('start to download file')
        self.exec_cmd_str(cmd_str)




import requests
import re
import urllib3
urllib3.disable_warnings()
import os
if os.name == "nt":  # windows
    from urllib3.exceptions import InsecureRequestWarning
    urllib3.disable_warnings(InsecureRequestWarning)


class HttpManager:
    _headers = {}

    def __init__(self
                 , proxy
                 , max_hourly_page_view
                 , default_header
                 , log
                 , timeout):
        self.log = log
        self.proxy = proxy

        self._maxHourlyPageView = int(max_hourly_page_view)
        self.out_time = timeout

        # Take notes for every request for every url
        self.retry_times = dict()
        # self._requestTimeOut = timeout
        # self._traffic = 0
        # self._trafficInterval = 100
        # self._lastPoolIndex = 0
        # self._carrier = carrier
        # self._API_Interval = 30

        self.encoding = None

        self._headers = dict()
        self._cookies = dict()

        if default_header:
            self._init_header()

    def set_header(self, key, val):
        key = re.sub(r'//\d', '', key)
        self._headers[key] = val

    def download_page(self, url, **kwargs):
        try:
            post_data = kwargs.get("post_data")
            if post_data:  # post
                resp = self.http_post(url, post_data=post_data)
            else:  # get
                resp = self.http_get(url)

            if not resp or resp.status_code != 200:
                self.log.warn("response error. ", str(resp.status_code))
            return resp

        except Exception as e:
            self.log.warn("Exception DownloadPage"
                              , "url: [%s] mes: [%s]" % (url, str(e)))

    def _init_header(self):
        self.set_header("Proxy-Connection",
                        "keep-alive")
        self.set_header("User-Agent",
                        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36")
        self.set_header("Accept",
                        "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8")
        self.set_header("Accept-Language",
                        "en-US")
        self.set_header("Accept-Encoding",
                        "gzip, deflate")
        self.set_header("Content-Type",
                        "text/html; charset=utf-8")

    def http_get(self, url, payload=None, verify_cert=False):
        resp = requests.get(url
                            , proxies=self.proxy.using_proxy
                            , headers=self._headers
                            , cookies=self._cookies
                            , params=payload
                            , timeout=self.out_time
                            , verify=verify_cert)
        if self.encoding:
            resp.encoding = self.encoding
        return resp

    def http_post(self, url, post_data, verify_cert=False):
        resp = requests.post(url
                             , proxies=self.proxy.using_proxy
                             , headers=self._headers
                             , cookies=self._cookies
                             , data=post_data
                             , timeout=self.out_time
                             , verify=verify_cert)
        if self.encoding:
            resp.encoding = self.encoding
        return resp


from Core.DaoBase import DaoBase


class TestDao(DaoBase):
    def __init__(self, **kwargs):
        super(TestDao, self).__init__(**kwargs)
        self.connect_to_mongo()


    def save(self, source_block):
        self.save_to_mongo(source_block)

    def save_to_mongo(self, data_block):
        self.db= self.mongo_client['GoogleMapPopularity']
        self.collection = self.db['data']
        self.collection.insert_one(data_block)



import inspect
import logging


class StructuredMessage(object):
    """docstring for StructuredMessage"""

    def __init__(self, subject, detail):
        super(StructuredMessage, self).__init__()
        self.subject = subject
        self.detail = detail

    def __str__(self):
        return '{}  {}'.format(self.subject, self.detail)


class Log(object):
    def __init__(self, log_name):
        self.log_name = log_name
        self.logger = self.init_log()
        self._ = StructuredMessage

    def init_log(self):
        logger = logging.Logger(self.log_name)
        logger.setLevel(logging.DEBUG)

        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        fh = logging.FileHandler('Log/' + self.log_name + '.log')
        fh.setFormatter(formatter)
        # todo add json part of log later, for elk search
        ch = logging.StreamHandler()
        ch.setFormatter(formatter)
        logger.addHandler(fh)
        logger.addHandler(ch)
        return logger

    def info(self, subject, detail=''):
        self.log_dynamic_information()
        self.logger.info(self._(subject, detail))

    def error(self, subject, detail=''):
        self.log_dynamic_information()
        self.logger.error(self._(subject, detail))

    def warn(self, subject, detail=''):
        self.log_dynamic_information()
        self.logger.warn(self._(subject, detail))

    def log_dynamic_information(self):
        parent = inspect.stack()
        frame, file_path, line, method, _, _ = parent[3]
        source_file = file_path.split("\\")[-1]
        cls_name = frame.f_locals.get("self").__class__.__name__
        print('{} - {} - {} -{}'.format(source_file, line, cls_name, method))
